<?php

namespace BeycanPress\Licensor\Models;

use Beycan\Moodel\AbstractModel;

/**
 * License table model
 */
class License extends AbstractModel 
{
    public $version = '1.0.3';

    public function __construct()
    {
        /**
         * licenseType enums
         * BeycanPress\Licensor\Settings::get('licenseTypeEnums')
         * 
         * 
         * placeOfPurchase enums
         * 1 - CodeCanyon
         * 2 - My store
         * 
         * 
         * status enums
         * 1 - Active
         * 2 - Passive
         */
        parent::__construct([
            'productId' => [
                'type' => 'bigint',
            ],
            'product' => [
                'type' => 'string',
                'length' => 150,
            ],
            'productCode' => [
                'type' => 'string',
                'length' => 150,
                'nullable' => true
            ],
            'licenseCode' => [
                'type' => 'string',
                'length' => 36,
                'index' => [
                    'type' => 'unique'
                ]
            ],
            'licenseType' => [
                'type' => 'integer',
            ],
            'domain' => [
                'type' => 'string',
                'length' => 150,
                'nullable' => true
            ],
            'earning' => [
                'type' => 'float',
            ],
            'support' => [
                'type' => 'float',
            ],
            'buyer' => [
                'type' => 'string',
                'length' => 70,
            ],
            'email' => [
                'type' => 'string',
                'length' => 100,
                'nullable' => true
            ],
            'placeOfPurchase' => [
                'type' => 'integer',
            ],
            'status' => [
                'type' => 'integer',
            ],
            'soldAt' => [
                'type' => 'datetime',
            ],
            'subId' => [
                'type' => 'bigint',
                'nullable' => true
            ],
            'orderId' => [
                'type' => 'bigint',
                'nullable' => true
            ],
            'expireTime' => [
                'type' => 'string',
                'length' => 100,
                'nullable' => true
            ],
            'createdAt' => [
                'type' => 'timestamp',
                'default' => 'current_timestamp',
            ],
        ]);
    }
    
    public function search(string $text) : array
    {
        return $this->getResults(str_ireplace(
            '%s', 
            '%' . $this->db->esc_like($text) . '%', "
            SELECT * FROM {$this->tableName} 
            WHERE productId LIKE '%s' 
            OR product LIKE '%s' 
            OR licenseCode LIKE '%s'
            OR productCode LIKE '%s'
            OR licenseType LIKE '%s'
            OR domain LIKE '%s'
            OR earning LIKE '%s'
            OR support LIKE '%s'
            OR buyer LIKE '%s'
            OR email LIKE '%s'
            OR placeOfPurchase LIKE '%s'
        "));
    }

    public function searchCount(string $text) : int
    {
        return (int) $this->getVar(
            str_ireplace(
            '%s', 
            '%' . $this->db->esc_like($text) . '%', "
            SELECT COUNT(id) FROM {$this->tableName} 
            WHERE productId LIKE '%s' 
            OR product LIKE '%s' 
            OR licenseCode LIKE '%s'
            OR productCode LIKE '%s'
            OR licenseType LIKE '%s'
            OR domain LIKE '%s'
            OR earning LIKE '%s'
            OR support LIKE '%s'
            OR buyer LIKE '%s'
            OR email LIKE '%s'
            OR placeOfPurchase LIKE '%s'
        "));
    }
}