<?php

namespace BeycanPress\Licensor\WooCommerce;

use BeycanPress\Licensor\LicenseCreator;
use BeycanPress\Licensor\Models\License;
use BeycanPress\Licensor\PluginHero\Helpers;

class LicenseProcess 
{
    use Helpers;

    /**
     * @var object
     */
    public static $currentUser;

    /**
     * @var object
     */
    public static $licenseModel;

    /**
     * @var int
     */
    private $subId = 0;

    public function __construct()
    {
        self::$currentUser = wp_get_current_user();
        self::$licenseModel = new License();

        // Domain change process
        add_action('woocommerce_before_add_to_cart_button', [$this, 'productCustomFields']);
        add_filter('woocommerce_add_to_cart_validation', [$this, 'productCustomFieldsValidation'], 10, 2);
        add_filter('woocommerce_add_cart_item_data', [$this, 'addToCartItemData'], 10);
        add_filter('woocommerce_get_item_data', [$this, 'showDataInCart'], 10, 2);
        add_action('woocommerce_checkout_create_order_line_item',[$this, 'updateOrderItemMeta'], 20, 3);
        add_filter('woocommerce_order_item_display_meta_key', [$this, 'orderItemDisplayMetaKey'], 10, 2);
        add_filter('woocommerce_order_item_display_meta_value', [$this, 'orderItemDisplayMetaValue'], 10, 2);
        add_action('woocommerce_single_product_summary', [$this, 'productSummary'], 30);
        add_filter('woocommerce_is_purchasable', [$this, 'isPurchasable'], 10, 2);
        add_filter('woocommerce_quantity_input_max', [$this, 'quantityInputMax'], 10, 2);

        // License process
        if ($this->setting('licenseInfoOrderPage')) {
            add_action('woocommerce_thankyou', [$this, 'licenseArea'], 1);
            add_action('woocommerce_view_order', [$this, 'licenseArea'], 1);
            //add_action('woocommerce_account_dashboard', [$this, 'licenseAreaByUser'], 1);
            add_action('woocommerce_subscription_details_table', [$this, 'licenseAreaSubs'], 12);
        }

        add_action('woocommerce_payment_complete', [$this, 'orderPaymentComplete'], 10);
        add_filter('woocommerce_subscription_periods', [$this, 'subsAddUnlimitedPeriods'], 10, 2);
        add_filter('woocommerce_subscription_lengths', [$this, 'subsAddUnlimitedLengths'], 10);
        //add_action('woocommerce_subscription_payment_failed', [$this, 'subsPaymentFailed'], 10);
        add_action('woocommerce_subscription_payment_complete', [$this, 'subsPaymentComplete'], 10);
        add_action('woocommerce_subscription_status_updated', [$this, 'subsStatusUpdate'], 10, 2);
        add_action('woocommerce_scheduled_subscription_payment', [$this, 'renewalPaymentProcess'], 10);

        
        new MetaBox();
        if ($this->setting('myLicensesPage')) {
            new MyAccount();
        }
    }

    /**
     * @param int $orderId
     * @return string
     */
    public function licenseArea(int $orderId) : void
    {
        $order = wc_get_order($orderId);

        if ($licenses = $order->get_meta('productLicenseCodes')) {
            $licenses = unserialize($licenses);
            if (!empty($licenses)) {
                $licenses = array_map(function ($license) {
                    return "'$license'";
                }, $licenses);
                $licenses = self::$licenseModel->findBy([['licenseCode', 'IN', $licenses]], ['id', 'DESC']);
            }
        }
        
        $this->viewEcho('license-area', ['licenses' => $licenses]);
    }

    /**
     * @param object $sub
     * @return void
     */
    public function licenseAreaSubs(object $sub)
    {
        $orderIds = array_values($sub->get_related_orders());
        $orderId = end($orderIds);
        $this->licenseArea($orderId);
    }

    /**
     * @return void
     */
    public function licenseAreaByUser()
    {
        $licenses = self::$licenseModel->findBy(['buyer' => self::$currentUser->user_login], ['id', 'DESC']);
        $this->viewEcho('license-area', ['licenses' => $licenses]);
    }

    /**
     * @param integer $orderId
     * @return void
     */
    public function orderPaymentComplete(int $orderId) : void
    {
        if (!self::$licenseModel->findOneBy(['subId' => $this->subId])) {
            $order = wc_get_order($orderId);
            $items = $order->get_items();
            $sub = $this->getOrderSub($orderId);
    
            $productLicenseCodes = [];
            foreach ($items as $item) {
                $licensorProductType = get_post_meta($item->get_product_id(), 'licensorProductType', true);
                if ($licensorProductType == 'licensed') {
                    for ($i = 0; $i < $item->get_quantity(); $i++) { 
                        $this->licensedProductProcess($item, $sub, $productLicenseCodes);
                    }
                } elseif ($licensorProductType == 'domain-change') {
                    $this->domainChangeProductProcess($item);
                }
            }
    
            $order->update_meta_data(
                'productLicenseCodes',
                serialize($productLicenseCodes)
            );
    
            $order->save();
        }
    }

    /**
     * @param object $item
     * @return void
     */
    private function domainChangeProductProcess(object $item) : void
    {
        $newDomain = $item->get_meta('licensorNewDomain');
        $licenseId = $item->get_meta('licensorLicenseId');

        self::$licenseModel->update(
            [
                'domain' => $this->parseDomain($newDomain)
            ],
            [
                'id' => $licenseId
            ]
        );

    }

    /**
     * @param object $item
     * @param object|null $subs
     * @param array $productLicenseCodes
     * @return void
     */
    private function licensedProductProcess(object $item, ?object $sub, array &$productLicenseCodes) : void
    {
        $productId = $item->get_product_id();
        
        $expireTime = null;
        if (class_exists('WC_Subscriptions_Product') && \WC_Subscriptions_Product::is_subscription($productId)) {
            if ($sub && \WC_Subscriptions_Product::get_period($productId) != 'unlimited') {
                $expireTime = $this->getSubExpireTime($sub);
            }
        }

        $licenseCode = sanitize_text_field(LicenseCreator::create());
                
        $metaDatas = array_values($item->get_all_formatted_meta_data());
        $licenseTypeKey = array_search('license-type', array_column($metaDatas, 'key'));
        $licenseTypeData = isset($metaDatas[$licenseTypeKey]) ? $metaDatas[$licenseTypeKey] : null;

        $licenseTypeEnums = $this->setting('licenseTypeEnums');
        if ($licenseTypeData !== null && $licenseTypeEnums) {;
            $licenseTypeEnumIndex = array_search($licenseTypeData->value, array_column($licenseTypeEnums, 'licenseType'));
            $licenseType = (int) $licenseTypeEnums[$licenseTypeEnumIndex]['number'] ?? 1;
        } else {
            $licenseType = 1;
        }

        $productLicenseCodes[] = $licenseCode;

        self::$licenseModel->insert([
            'productId' => $productId,
            'product' => $item->get_name(),
            'productCode' => null,
            'licenseCode' => $licenseCode,
            'licenseType' => $licenseType,
            'domain' => null,
            'earning' => $item->get_total(),
            'support' => 0,
            'buyer' => self::$currentUser->user_login,
            'email' => self::$currentUser->user_email,
            'placeOfPurchase' => 2,
            'status' => 1,
            'subId' => isset($sub) ? $sub->get_id() : null,
            'orderId' => $item->get_order_id(),
            'soldAt' => date('Y-m-d H:i:s'),
            'expireTime' => $expireTime,
        ]);
    }

    /**
     * @return void
     */
    public function productCustomFields() : void
    {
        global $post;
        $licensorProductType = get_post_meta($post->ID, 'licensorProductType', true);

        if ($licensorProductType == 'domain-change') {
            $licenses = self::$licenseModel->findBy(['buyer' => self::$currentUser->user_login], ['id', 'DESC']);
    
            $this->viewEcho('custom-fields', [
                'licenses' => $licenses
            ]);
        }
    }

    /**
     * @param boolean $passed
     * @param integer $productId
     * @return boolean
     */
    public function productCustomFieldsValidation(bool $passed, int $productId) : bool
    {
        $licensorProductType = get_post_meta($productId, 'licensorProductType', true);

        if ($licensorProductType == 'domain-change') {
            if ((!isset($_POST['licensorLicenseId']) || $_POST['licensorLicenseId'] == '') || (!isset($_POST['licensorNewDomain']) || $_POST['licensorNewDomain'] == '')) {
                wc_add_notice(
                    esc_html__( 'Please fill in the product fields before adding to cart.', 'weasyfields' ),
                    'error'
                );
                return false;
            }
        }

        return $passed;
    }

    /**
     * @param array $cartItem 
     * @return array $cartItem 
     */
    public function addToCartItemData(array $cartItem) : array
    {
        // Is there any post data sent for me?
        if (isset($_POST['licensorLicenseId'])) {
            $cartItem['licensorLicenseId'] = $_POST['licensorLicenseId'];
            $cartItem['licensorNewDomain'] = $this->parseDomain($_POST['licensorNewDomain']);
        }

        // Avoid merging items
        $cartItem['unique_key'] = md5(microtime().rand());

        return $cartItem;
    }

    /**
     * @param array $cartItemData 
     * @param array $cartItem data 
     * @return array $cartItemData
     */
    public function showDataInCart(array $cartItemData, array $cartItem) : array
    {
        if (isset($cartItem['licensorLicenseId'])) {
            $license = self::$licenseModel->findOneBy(['id' => $cartItem['licensorLicenseId']]);
            $cartItemData[] = [
                'name'  => esc_html__('License', 'licensor'),
                'value' => $license->product
            ];
            $cartItemData[] = [
                'name'  => esc_html__('New domain', 'licensor'),
                'value' => $cartItem['licensorNewDomain']
            ];
        }

        return $cartItemData;
    }
    
    /**
     * @param object $item
     * @param string $cartItemKey
     * @param array $values
     * @return void
     */
    public function updateOrderItemMeta(object $item, string $cartItemKey, array $values) : void
    {
        if (isset($values['licensorNewDomain'])) {
            $parsedDomain = $this->parseDomain($values['licensorNewDomain']);
            $item->update_meta_data('licensorLicenseId',  $values['licensorLicenseId']);
            $item->update_meta_data('licensorNewDomain',  $parsedDomain);
        }
    }

    /**
     * @param string $displayValue
     * @param object $meta
     * @return string
     */
    public function orderItemDisplayMetaKey(string $displayKey, object $meta) : string
    {
        if ($meta->key == 'licensorLicenseId') {
            $displayKey = esc_html__('License', 'licensor');
        } elseif ($meta->key == 'licensorNewDomain') {
            $displayKey = esc_html__('New domain', 'licensor');
        }

        return $displayKey;
    }

    /**
     * @param string $displayValue
     * @param object $meta
     * @return string
     */
    public function orderItemDisplayMetaValue(string $displayValue, object $meta) : string
    {
        if ($meta->key == 'licensorLicenseId') {
            $license = self::$licenseModel->findOneBy(['id' => $displayValue]);
            $displayValue = $license->product;
        }

        return $displayValue;
    }

    /**
     * @param array $periods
     * @param integer $number
     * @return array
     */
    public function subsAddUnlimitedPeriods(array $periods, int $number) : array
    {
        $periods['unlimited'] = sprintf(_nx('unlimited', '%s unlimited', $number, 'Subscription billing period.', 'woocommerce-subscriptions'), $number);

        return $periods;
    }
    
    /**
     * @param array $ranges
     * @return array
     */
    public function subsAddUnlimitedLengths(array $ranges) 
    {
        $ranges['unlimited'] = [
			_x('Never expire', 'Subscription length', 'woocommerce-subscriptions'),
        ];

        return $ranges;
    }

    /**
     * @param boolean $productPurchasable
     * @param object $product
     * @return boolean
     */
    public function isPurchasable(bool $productPurchasable, object $product) : bool
    {
        $licensorProductType = get_post_meta($product->get_id(), 'licensorProductType', true);

        if ($licensorProductType == 'domain-change') {
            $licenses = self::$licenseModel->findBy(['buyer' => self::$currentUser->user_login], ['id', 'DESC']);
            if (empty($licenses)) return false;
            
        }
    
        return $productPurchasable;
    }

    /**
     * @param int $maxQuantity
     * @param object $product
     * @return int
     */
    public function quantityInputMax(int $maxQuantity, object $product) : int
    {
        $licensorProductType = get_post_meta($product->get_id(), 'licensorProductType', true);

        if ($licensorProductType == 'domain-change') {
            return 1;
        }
    
        return $maxQuantity;
    }

    /**
     * @return void
     */
    public function productSummary() : void
    {
        global $product;
    
        $licensorProductType = get_post_meta($product->get_id(), 'licensorProductType', true);
        if ($licensorProductType == 'domain-change') {
            $licenses = self::$licenseModel->findBy(['buyer' => self::$currentUser->user_login], ['id', 'DESC']);
            if (empty($licenses)) esc_html_e('You can\'t buy this product right now because you don\'t have any licenses!', 'licensor');
            
        }
    }

    /**
     * @param object $sub
     * @return void
     */
    public function subsPaymentFailed(object $sub) : void
    {
        self::$licenseModel->update(['status' => 2], ['subId' => $sub->get_id()]);
    }

    /**
     * @param object $sub
     * @return void
     */
    public function subsPaymentComplete(object $sub) : void
    {
        $this->subId = $sub->get_id();
    }

    /**
     * @param object $sub
     * @param string $newStatus
     * @return void
     */
    public function subsStatusUpdate(object $sub, string $newStatus) : void
    {
        if ($newStatus == 'active') {
            self::$licenseModel->update([
                'status' => 1,
                'expireTime' => $this->getSubExpireTime($sub),
        ], ['subId' => $sub->get_id()]);
        } elseif (in_array($newStatus, ['cancelled', 'expired', 'on-hold'])) {
            self::$licenseModel->update(['status' => 2], ['subId' => $sub->get_id()]);
        }
    }

    /**
     * @param int $subId
     * @return void
     */
    public function renewalPaymentProcess(int $subId) : void
    {
        $this->subId = $subId;
    }

    /**
     * @param object $sub
     * @return string
     */
    private function getSubExpireTime(object $sub) : string
    {
        $expireTime = $sub ? $sub->get_date('end') : null;
        $nextPayment = $sub->get_time('next_payment');
        $trialEndDate = $sub->get_time('trial_end_date');
        $expireTime = $nextPayment ? date('Y-m-d H:i:s', $nextPayment) : $expireTime;
        if ($trialEndDate && $trialEndDate > time()) {
            $expireTime = date('Y-m-d H:i:s', ($trialEndDate - time()) + $nextPayment);
        }

        return $expireTime;
    }

    /**
     * @param int $orderId
     * @return object|null
     */
    private function getOrderSub(int $orderId) : ?object
    {
        if ($this->subId) {
            return wcs_get_subscription($this->subId);
        } else {
            $subs = wcs_get_subscriptions_for_order($orderId);
            return empty($subs) ? null : array_values($subs)[0];
        }
    }

    /**
     * @param integer $subId
     * @param array $dates
     * @return void
     */
    function changeSubDates(int $subId, array $dates) : void
    {
        $sub = wcs_get_subscription($subId);
        $sub->update_dates($dates);
    }
}
